//= require <gettexthelper>
//= require <messagebus>
//= require <statusbar>
//= require <navpanel>

Ext.namespace('dSS11', 'dSS11.config', 'dSS', 'dSA');

dSS11.CreateMainViewport = function(panel)
{
    /* for some reason this check is also triggered on IE9, need a fix in ExtJS
    if (Ext.isIE6 === true) {
        Ext.Msg.alert(_("Error"),
                      _("Internet Explorer 6 is not supported, please " +
                        "upgrade your webbrowser!"));
        return;
    }
    */

    var viewport = new Ext.Viewport(
    {
        hideBorders: true,
        layout: 'border',
        items:
            [
                {
                    xtype: 'box',
                    region: 'north',
                    margins: { 'top': 10, left: 10, right: 10 },
                    height: 50,
                    autoEl:
                    {
                        tag: 'div',
                        html: '<a href="http://digitalstrom.org/" alt="' +
                                _("visit digitalSTROM.org") +
                                '"><img id="logo" src="images/ds_logo.png"' +
                                ' alt="digitalSTROM Logo"></a>' +
                                '<h1><a href="http://digitalstrom.org/" ' +
                                'alt="visit digitalSTROM.org">' +
                                _("digitalSTROM Configurator") + '</a></h1>'
                    }
                },
                panel
            ]
    });

    if (dSS11.config.dSAIntegration === true) {
        dSA.config.language = dSS11.config.language;
        dSA.LoadLangpack(null,
            dSS11.StatusBar.setMessage.createDelegate(dSS11.StatusBar));
    }

    return viewport;
};

dSS11.createMainPanel = function(enableModule) {
    var listeners;

    var items =
    [
            dSS11.AddOnsPanel
    ];

    if (dSS11.config.dSSIntegration === true) {
        items = items.concat(
        [
            {
                xtype: 'dssscenespanel',
                title: _("SW: Scenes"),
                ref: 'scenes'
            },
            {
                xtype: 'dsszonebrowser',
                title: _("SW: Zones"),
                ref: 'zones'
            }
        ]);

    }

    items.push(
        {
            xtype: 'dss11navpanel',
            title: _("SW: System"),
            ref: 'system'
        }
    );


    if (dSS11.config.dSSIntegration === true) {
        items.push(
            {
                title: dSS._("Hardware"),
                xtype: 'dsmoverviewpanel',
                ref: 'circuits'
            }
        );
    }

    items.push(
        {
            title: _("Help"),
            ref: 'help',
            xtype: 'dss11help'
        }
    );

    var panel =
    {
        xtype: 'tabpanel',
        region: 'center',
        ref: 'dSS11Main',
        activeTab: 0,
        items: items,
        bbar: dSS11.StatusBar,
        listeners: {
            afterrender: function() {
                if (dSS11.config.dSSIntegration === true) {
                    this.hideTabStripItem(this.circuits);
                }
                this.hideTabStripItem(this.system);

                if (dSS11.config.dSSIntegration === true) {
                    dSS11.GetDSSVersion();
                }
            },
            refreshview: function() {
                var active = this.getActiveTab();
                if (dSS11.config.dSSIntegration === true) {
                    if ((active === this.circuits) ||
                        (active === this.scenes) ||
                        (active === this.zones)) {
                        dSS.MessageBus.fireEvent('reloadapartment');
                        dSS.config.firstLoad = false;
                    }
                }

                if (active === this.addons) {
                    dSS11.MessageBus.fireEvent('refreshaddonsview');
                } else if (active === this.system) {
                    dSS11.MessageBus.fireEvent('refreshsystemupdateview');
                }
            },
            switchview: function(advanced) {
                if (advanced === true) { // go advanced
                    if (dSS11.config.dSSIntegration === true) {
                        this.unhideTabStripItem(this.circuits);
                    }
                    this.unhideTabStripItem(this.system);
                } else { // go default
                    var active = this.getActiveTab();

                    if (dSS11.config.dSSIntegration === true) {
                        if ((active === this.system) ||
                            (active === this.circuits)) {
                            this.setActiveTab(this.scenes);
                        }

                        this.hideTabStripItem(this.circuits);
                    } else {
                        if (active === this.system) {
                            this.setActiveTab(0);
                        }
                    }

                    this.hideTabStripItem(this.system);
                }
            } // switchview
        } // listeners
    };


    return panel;
};

// this is an ugly hack, we will try to keep our dSS session active so that
// it's "security" mechanism does not kick us out too early. We will also
// re-login in case we managed to time out somehow (for instance if dSS was
// restarted). This function will never return any errors and will be triggered
// once in a minute.
dSS11.SessionPoller = function() {
    Ext.Ajax.request({
        url: "/json/system/version",
        disableCaching: true,
        success: function(response, options) {
            dSS11.SessionPoller.defer(dSS11.config.dSSPollInterval, this);
        },
        failure: function(response, options) {
            if (response.status === 403)
            {
                Ext.Ajax.request({
                    url: '/json/system/login',
                    params:
                    {
                        user: 'dssadmin',
                        password: 'dssadmin'
                    },
                    scope: this,
                    method: "GET",
                    disableCaching: true,
                    success: function(response, options) {
                        dSS11.SessionPoller.defer(dSS11.config.dSSPollInterval,
                                                  this);
                    },
                    failure: function(response, options) {
                        dSS11.SessionPoller.defer(dSS11.config.dSSPollInterval,
                                                  this);
                    }
                });
            } else {
                dSS11.SessionPoller.defer(dSS11.config.dSSPollInterval, this);
            }
        }
    });
};

dSS11.GetDSSVersion = function() {
    Ext.Ajax.request({
         url: "/json/system/version",
         disableCaching: true,
         success: function(response, options) {
             try {
                 var data = Ext.decode(response.responseText);
                 if (data.ok === true) {

                     dSS11.StatusBar.setVersion(data.result.version);
                     return;
                 }

                 dSS11.StatusBar.setMessage(_("Could not get dSS version"));
             } catch(error) {
                 dSS11.StatusBar.setMessage(_("Could not get dSS version") +
                                            ': ' + error);
             }
         },
         failure: function(response, options) {
             dSS11.StatusBar.setMessage(_("Could not get dSS version"));
         }
     });
};

dSS11.dSSLogin = function()
{
    // initialize dSS gettext
    Ext.Ajax.request({
        url: '/json/system/login',
        params:
        {
            user: 'dssadmin',
            password: 'dssadmin'
        },
        scope: this,
        method: "GET",
        disableCaching: true,
        success: function(response, options) {
            try {
                if (response.responseText.length === 0) {
                    dSS11.config.dSSIntegration = false;
                    Ext.Msg.alert(_("Error"),
                        _("No response from dSS, proceeding in limited mode." +
                          " Check if the dSS is running and reload this page " +
                          "again!"));
                    dSS11.StatusBar.setMessage(_("No response from dSS, please check if dSS is running"));
                    this.viewport = dSS11.CreateMainViewport(
                                                dSS11.createMainPanel());
                    return;

                }

                var jsonData = Ext.util.JSON.decode(response.responseText);
                if (jsonData.ok !== true) {
                    dSS11.config.dSSIntegration = false;
                    Ext.Msg.alert(_("Error"),
                        _("Could not log in, dSS access disabled"));
                    dSS11.StatusBar.setMessage(_("Could not log in, dSS " +
                            "access disabled! Proceeding in limited mode."));
                    this.viewport = dSS11.CreateMainViewport(
                                                dSS11.createMainPanel());
                    return;
                }
                dSS11.SessionPoller.defer(dSS11.config.dSSPollInterval, this);
                this.viewport = dSS11.CreateMainViewport(
                                                dSS11.createMainPanel());
            } catch (error) {
                dSS11.config.dSSIntegration = false;
                Ext.Msg.alert(_("Error"),
                              _("Could not log in, dSS access disabled") +
                              ': ' + error.message);
                dSS11.StatusBar.setMessage(_("Could not log in, dSS access " +
                                    "disabled! Proceeding in limited mode."));
                this.viewport = dSS11.CreateMainViewport(
                                                dSS11.createMainPanel());
            }
        },
        failure: function(response, options) {
            dSS11.config.dSSIntegration = false;
            Ext.Msg.alert(_("Error"),
                    _("Could not log in, dSS access disabled"));
            dSS11.StatusBar.setMessage(_("Could not log in, dSS access " +
                                    "disabled! Proceeding in limited mode."));
            this.viewport = dSS11.CreateMainViewport(
                                                dSS11.createMainPanel());
        }
    });
};

dSS11.loadDSSLangpack = function()
{
    Ext.Ajax.request({
        url: dSS.config.langPath,
        scope: this,
        method: "GET",
        disableCaching: true,
        success: function(response, options) {
            try {
                var data = Ext.util.JSON.decode(response.responseText);

                var params = {
                                'domain' : 'dss-installer',
                                'locale_data' : { 'dss-installer' : data }
                             };
                dSS.gt = new Gettext(params);
                dSS11.dSSLogin();
            } catch(err) {
                Ext.Msg.alert(_("Error"),
                              _("Could not load language package") + ': ' +
                              err);
                dSS11.dSSLogin();
            }
        },
        failure: function(response, options) {
            Ext.Msg.alert(_("Error"),
                          _("Could not load language packge"));
            dSS11.dSSLogin();
        }
    });
};

dSS11.localizeExt = function()
{
    Ext.MessageBox.buttonText.yes = _("Yes");
    Ext.MessageBox.buttonText.no = _("No");
    Ext.MessageBox.buttonText.cancel = _("Cancel");
};

Ext.onReady(
    function()
    {
        Ext.get('start').remove();
        dSS11.config.dSS11TimeDiff = 0;
        dSS11.config.timeUpdateTimeout = 0;
        if (dSS.module === true) {
            dSS11.config.dSSIntegration = true;
        } else {
            dSS11.config.dSSIntegration = false;
        }

        if (dSA.module === true) {
            dSS11.config.dSAIntegration = true;
        } else {
            dSS11.config.dSAIntegration = false;
        }

        dSS11.config.language = 'de_DE';
        dSS11.config.langPath = '/locale/' + dSS11.config.language +
                                '/LC_MESSAGES/';

        dSS11.config.rebootWait = 70; // seconds to wait for reboot
        dSS11.config.codes = {
            "ERR_OK"        : 0,
            "ERR_MODE"      : 1,
            "ERR_MASK"      : 2,
            "ERR_IP"        : 3,
            "ERR_GW"        : 4,
            "ERR_DNS"       : 5,
            "ERR_FWRITE"    : 6,
            "ERR_NETDOWN"   : 7,
            "ERR_NETUP"     : 8,
            "ERR_PARAM"     : 9,
            "ERR_REDIRECT"  : 10,
            "ERR_RESTORE"   : 11,
            "ERR_BACKUP"    : 12,
            "ERR_HOSTNAME"  : 13,
            "ERR_APPSTART"  : 14,
            "ERR_REBOOT"    : 15,
            "ERR_HALT"      : 16,
            "ERR_NOZONES"   : 17,
            "ERR_NONTP"     : 18,
            "ERR_NONTPSRV"  : 19,
            "ERR_LINK"      : 20,
            "ERR_DATE"      : 21,
            "ERR_TIME"      : 22,
            "ERR_PASSWORD"  : 23,
            "ERR_SETUP"     : 24,
            "ERR_DOWAIT"    : 25,
            "ERR_UPDATE"    : 26,
            "ERR_UPNPNAME"  : 27,
            "ERR_DSMFW"     : 28,
            "ERR_DISCLAIMER": 29
        };

        dSS11.config.opkgstate = {
            "OPKG_NOT_INSTALLED"    : 1,
            "OPKG_UNPACKED"         : 2,
            "OPKG_HALF_CONFIGURED"  : 3,
            "OPKG_INSTALLED"        : 4,
            "OPKG_HALF_INSTALLED"   : 5,
            "OPKG_CONFIG_FILES"     : 6,
            "OPKG_POST_INST_FAILED" : 7,
            "OPKG_REMOVAL_FAILED"   : 8,
            "OPKG_LAST_STATE_STATUS": 9
        };

        dSS11.config.apartmentid = {
            "MIN" : 0,
            "MAX" : 62
        };

        dSS11.config.dSSPollInterval = 2*60000; // ms, poll once in two minutes

        // automatically reload UI after update of following packages
        dSS11.config.reloadUIAfterUpdate =
        [
            'dss11-websetup', 'dss-web-module', 'dsa-ui-module', 'dss'
        ];

        Ext.QuickTips.init();

        Ext.Ajax.request({
            url            : dSS11.config.langPath + 'langpack.json',
            disableCaching : true,
            scope          : this,
            success        : function(response, options) {
                try{
                    var data = Ext.decode(response.responseText);
                    var params = {
                                    "domain" : "dss11",
                                    "locale_data" : { "dss11" : data }
                                 };

                    dSS11.config.gt = new Gettext(params);

                    dSS11.localizeExt();

                    if (dSS11.config.dSSIntegration === true)  {
                        dSS11.loadDSSLangpack();
                    } else {
                        this.viewport = dSS11.CreateMainViewport(
                                                dSS11.createMainPanel());
                    }
                } catch(error) {
                    Ext.Msg.alert(_("Error"),
                            _("Could not load language package") + ': ' +
                            error.message);

                    if (dSS11.config.dSSIntegration === true)  {
                        dSS11.loadDSSLangpack();
                    } else {
                        this.viewport = dSS11.CreateMainViewport(
                                                dSS11.createMainPanel());
                    }
                }
            },
            failure : function(response, options) {
                Ext.Msg.alert(_("Error"), _("Could not load language package"));
                if (dSS11.config.dSSIntegration === true)  {
                    dSS11.loadDSSLangpack();
                } else {
                    this.viewport = dSS11.CreateMainViewport(
                                            dSS11.createMainPanel());
                }
            }
        });
    }
);

